/*---------------+---------------------------------------------------------------+--------------+
                 | Author: Daniel Porumbel 2018       daniel.porumbel@cnam.fr    |
                 |License: Creative Commons Attribution 3.0 Unported License     |
                 |         http://creativecommons.org/licenses/by/3.0/           |             
                 +--------------------------------------------------------------*/
#ifndef GRSTABLES_H_INCLUDED
#define GRSTABLES_H_INCLUDED
extern int**   edge;           //These variables simply define the graph:
extern int     n;              //vtx no
extern int     m;              //edge no
#define NO_CPLEX_OUTPUT

//find the maximum w-weigted stable. Normal version uses binary=1. LP Relaxation
//uses binary =0. Returns the weighted value or -heuristicWeightVal if not enough time
//If binary=1, record the stable in stable
//If binary>1, then use at most binary vertices
//Timeout in seconds. It can be doubled if heur_stabMax<1 (not cutting curr sol)
//It can return a -heuristicStable is timeout reaached.
double maxWeightStableCplex(int* stable, double * w, int binary, int timeout);
double maxWeightCliqueCplex(int* stable, double * w, int binary, int timeout);

//A heuristic that returns the best clique using weights w with max maxCardinal elems.
//This function start trying cliques with maxCardinal, then maxCardinal - 1, ...  minCardinal
//Returns the best clique and records it in array clique
//If minCardinal is missing, then minCardinal=0
double maxWeightCliqueLocalSearch(int* clique, double * w, int maxCardinal);
double maxWeightCliqueLocalSearch(int* clique, double * w, int maxCardinal, int minCardinal);

//reads a coloring file containing: NBcolors, stableSizes, stables
//return 1 if successful
int readColFile(char* filename, int**& stables, int& colors);

int fileExists(char* filename);
void writeArray(int nbVals, double* vec, char* filename, int precision);
void readArray(int nbVals, double* vec, char* filename);

//read a graph from DIMACS format filename
void readGraph(char*filename);

//generate a simple graph with n=20
void readGraph();

//put some testing here
void testMaxStable();

//Print a matrix on the output
void printMat(int**mat,int nn);
#endif
