/*---------------+---------------------------------------------------------------+--------------+
                 | Author: Daniel Porumbel 2018       daniel.porumbel@cnam.fr    |
                 |License: Creative Commons Attribution 3.0 Unported License     |
                 |         http://creativecommons.org/licenses/by/3.0/           |             
                 +--------------------------------------------------------------*/
#ifndef FILE_DIJKSTRA
#define FILE_DIJKSTRA
#include <vector>
#include <list>
#include <string>
using namespace std;


typedef double weight_t;                            //change here if you want to use ints
void setGraphSize(int);                             //you might call freeMem before
void addEdge(int i, int j, weight_t weight);

void dijkstraCalcPaths(int src,weight_t*distances);//fill the min distances
void getVtxOnPathTo(int dst, int* vertices);       //fill vertices with n 0/1 values
void dijkstraFreeMem();



/*********************************************************************************
//                                 USAGE EXAMPLE                                
int main(){
    setGraphSize(6);
    addEdge(0,1,10.1);
    addEdge(0,2,2000);
    addEdge(2,3,100);
    addEdge(1,4,5);
    addEdge(1,5,51);
    addEdge(4,5,6);
    double* distances = new double[6];
    dijkstraCalcPaths(0,distances);
    for(int i=0;i<6;i++)
        cout<<"i="<<i<<":"<<distances[i]<<endl;
    int * vertices = new int[6];
    getVtxOnPathTo(5,vertices);
    cout<<"Vertices on the shortest path to 5:";
    for(int i=0;i<6;i++)
        cout<<vertices[i]<<" ";
    cout<<endl;
    freeMem();
}
*********************************************************************************/





/********************************************************************************/
/*                          Methods using STD vectors                           */
void dijkstraCalcPaths(int src, vector<weight_t> &min_dist, vector<int> &previous);

list<int> dijkstraGetShPathTo( int vertex, const vector<int> &previous);

string dijkstraGetStrShPathTo( int vertex, const vector<int> &previous);


//Specific to exploiting sparsity
void removeLastVt();

//return the minimum value of the new links and adds all links that are >INT_MIN
double linkLastVtx(double*);
#endif
